##Chapter 2
#install.packages("ggplot2")
library(ggplot2)

bank_train <- read.csv("bank_marketing_training.csv")

table(bank_train$response, bank_train$previous_outcome)

t1 <- table(bank_train$response, bank_train$previous_outcome)

t1

bank_train[1, ]

bank_train[c(1, 3, 4), ]

bank_train[, c(1, 3)]

bank_train$age



##Chapter 3
n <- dim(bank_train)[1]

bank_train$Index <- c(1:n)

head(bank_train)

bank_train$days_since_previous <- ifelse(test = bank_train$days_since_previous == 999, 
                                         yes = NA, 
                                         no = bank_train$days_since_previous)
hist(bank_train$days_since_previous, xlab = "days_since_previous", main = "Histogram of days_since_previous - Missing Values replaced by NA")

#install.packages("plyr")
library(plyr)

edu.num <- revalue(x = bank_train$education, replace = c("illiterate" = 0, "basic.4y" = 4, "basic.6y" = 6, "basic.9y" = 9,
                                                         "high.school" = 12, "professional.course" = 12,
                                                         "university.degree" = 16, "unknown" = NA))

bank_train$education_numeric <- as.numeric(levels(edu.num))[edu.num]

bank_train$age_z <- scale(x = bank_train$age)

bank_outliers <- bank_train[which(bank_train$age_z < -3 |
                                    bank_train$age_z > 3), ]

bank_train_sort <- bank_train[order(-bank_train$age_z), ]

bank_train_sort[1:10, ]

head(bank_train_sort)

bank_train_sort[1:10, c(1, 3)]



##Chapter 4
library(ggplot2)

ggplot(bank_train, aes(previous_outcome)) + geom_bar() + coord_flip()

ggplot(bank_train, aes(previous_outcome)) + geom_bar(aes(fill = response)) + coord_flip()

ggplot(bank_train, aes(previous_outcome)) + geom_bar(aes(fill = response), position = "fill") + coord_flip()

t.v1 <- table(bank_train$response, bank_train$previous_outcome)

t.v2 <- addmargins(A = t.v1, FUN = list(total = sum), quiet = TRUE)

t.v2

round(prop.table(t.v1, margin = 2)*100, 1)

ggplot(bank_train, aes(age)) + geom_histogram(color = "black")

ggplot(bank_train, aes(age)) + geom_histogram(aes(fill = response), color = "black")

ggplot(bank_train, aes(age)) + geom_histogram(aes(fill = response), color = "black", position = "fill")

bank_train$age_binned <- cut(x = bank_train$age, breaks = c(0, 27, 60.01, 100), right = FALSE, labels = c("Under 27", "27 to 60", "Over60"))

ggplot(bank_train, aes(age_binned)) + geom_bar(aes(fill = response)) + coord_flip()

t2 <- table(bank_train$response, bank_train$age_binned)
t2
round(prop.table(t2, margin = 2)*100, 1)



##Chapter 5
bank <- read.csv("bank-additional.csv")

set.seed(7)

n <- dim(bank)[1]

train_ind <- runif(n) < 0.75

bank_train <- bank[train_ind, ]
bank_test <- bank[!train_ind, ]

table(bank_train$response)

to.resample <- which(bank_train$response == "yes")

our.resample <- sample(x = to.resample, size = 850, replace = TRUE)

our.resample <- bank_train[our.resample, ]

train_bank_rebal <- rbind(bank_train, our.resample)

t.v1 <- table(train_bank_rebal$response)
t.v2 <- rbind(t.v1, round(prop.table(t.v1), 4))
colnames(t.v2) <- c("Response = No", "Response = Yes")
rownames(t.v2) <- c("Count", "Proportion")
t.v2



##Chapter 6
adult_tr <- read.csv("adult_ch6_training.csv")

colnames(adult_tr)[1] <- "maritalStatus"

adult_tr$Income <- factor(adult_tr$Income)
adult_tr$maritalStatus <- factor(adult_tr$maritalStatus)

#install.packages(c("rpart", "rpart.plot"))
library(rpart)
library(rpart.plot)

cart01 <- rpart(formula = Income ~ maritalStatus + Cap_Gains_Losses, data = adult_tr, method = "class")

rpart.plot(cart01)

?rpart.plot

rpart.plot(cart01, type = 4, extra = 107)

X <- data.frame(maritalStatus = adult_tr$maritalStatus, Cap_Gains_Losses = adult_tr$Cap_Gains_Losses)

predIncomeCART <- predict(object = cart01, newdata = X, type = "class")

#install.packages("C50")
library(C50)

C5 <- C5.0(formula = Income ~ maritalStatus + Cap_Gains_Losses, data = adult_tr, control = C5.0Control(minCases = 75))

plot(C5)

predict(object = C5, newdata = X)

#install.packages("randomForest")
library(randomForest)

rf01 <- randomForest(formula = Income ~ maritalStatus + Cap_Gains_Losses, data = adult_tr, ntree = 100, type = "classification")

rf01$predicted



##Chapter 7
clothing_train <- read.csv("clothing_data_driven_training.csv")
clothing_test <- read.csv("clothing_data_driven_test.csv")

library(C50)

clothing_train$Response <- factor(clothing_train$Response)
C5 <- C5.0(Response ~ Days.since.Purchase + Number.of.Purchase.Visits + Sales.per.Visit, data = clothing_train)

test.X <- subset(x = clothing_test, select = c("Days.since.Purchase", "Number.of.Purchase.Visits", "Sales.per.Visit"))

ypred <- predict(object = C5, newdata = test.X)

t1 <- table(clothing_test$Response, ypred)
row.names(t1) <- c("Actual: 0", "Actual: 1")
colnames(t1) <- c("Predicted: 0", "Predicted: 1")
t1 <- addmargins(A = t1, FUN = list(Total = sum), quiet = TRUE)
t1

cost.C5 <- matrix(c(0, 4, 1, 0), byrow = TRUE, ncol = 2)
dimnames(cost.C5) <- list(c("0", "1"), c("0", "1"))

C5.costs <- C5.0(Response ~ Days.since.Purchase + Number.of.Purchase.Visits + Sales.per.Visit, data = clothing_train, costs = cost.C5)



##Chapter 8
wine_tr <- read.csv("wine_flag_training.csv")
wine_test <- read.csv("wine_flag_test.csv")

ta <- table(wine_tr$Type, wine_tr$Alcohol_flag)
colnames(ta) <- c("Alcohol = High", "Alcohol = Low")
rownames(ta) <- c("Type = Red", "Type = White")
addmargins(A = ta, FUN = list(Total = sum), quiet = TRUE)

ts <- table(wine_tr$Type, wine_tr$Sugar_flag)
colnames(ts) <- c("Sugar = High", "Sugar = Low")
rownames(ts) <- c("Type = Red", "Type = White")
addmargins(A = ts, FUN = list(Total = sum), quiet = TRUE)

#install.packages("gridExtra")
library(gridExtra)
library(ggplot2)

plot1 <- ggplot(wine_tr, aes(Type)) + geom_bar( aes(fill = Alcohol_flag), position = "fill") + ylab("Proportion")
plot2 <- ggplot(wine_tr, aes(Type)) + geom_bar( aes(fill =  Sugar_flag), position = "fill") + ylab("Proportion")
grid.arrange(plot1, plot2, nrow = 1)

#install.packages("e1071")
library(e1071)

nb01 <- naiveBayes(formula = Type ~ Alcohol_flag +  Sugar_flag, data = wine_tr)
nb01

ypred <- predict(object = nb01, newdata = wine_test)
t.preds <- table(wine_test$Type, ypred)
rownames(t.preds) <- c("Actual: Red", "Actual: White")
colnames(t.preds) <- c("Predicted: Red", "Predicted: White")
addmargins(A = t.preds, FUN = list(Total = sum), quiet = TRUE)



##Chapter 9
fram_train <- read.csv("Framingham_Training.csv")
fram_train$Death <- as.factor(fram_train$Death)
fram_train$Sex <- as.factor(fram_train$Sex)

fram_train$Age.mm <- (fram_train$Age -  min(fram_train$Age)) / (max(fram_train$Age) -   min(fram_train$Age))

#install.packages("nnet")
#install.packages("NeuralNetTools")
library(nnet)
library(NeuralNetTools)

nnet01 <- nnet(Death ~ Sex + Age.mm, data = fram_train,  size = 1)

plotnet(nnet01)

nnet01$wts



##Chapter 10
wine_train <- read.csv("white_wine_training.csv")

X <- subset(wine_train, select = c("alcohol", "sugar"))

Xs <- as.data.frame(scale(X))
colnames(Xs) <- c("alcohol_z", "sugar_z")

kmeans01 <- kmeans(Xs, centers = 2)

cluster <- as.factor(kmeans01$cluster)

Cluster1 <- Xs[which(cluster == 1), ]
Cluster2 <- Xs[which(cluster == 2), ]

summary(Cluster1)
summary(Cluster2)

wine_test <- read.csv("white_wine_test.csv")
X_test <- subset(wine_test, select = c("alcohol", "sugar"))
Xs_test <- as.data.frame(scale(X_test))
colnames(Xs_test) <- c("alcohol_z", "sugar_z")
kmeans01_test <- kmeans(Xs_test, centers = 2)
cluster_test <- as.factor(kmeans01_test$cluster)
Cluster1_test <- Xs[ which(cluster_test == 1), ]
Cluster2_test <- Xs[ which(cluster_test == 2), ]
summary(Cluster1_test)
summary(Cluster2_test)



##Chapter 11
sales_train <- read.csv("clothing_sales_training.csv")
sales_test <- read.csv("clothing_sales_test.csv")

sales_train$CC <- as.factor(sales_train$CC)
sales_train$Web <- as.factor(sales_train$Web)
sales_test$CC <- as.factor(sales_test$CC)
sales_test$Web <- as.factor(sales_test$Web)

model01 <- lm(Sales.per.Visit ~ Days + Web + CC, data = sales_train)

summary(model01)

model01_test <- lm(Sales.per.Visit ~ Days +Web + CC, data = sales_test)

summary(model01_test)

model02 <- lm(formula = Sales.per.Visit ~ Days + CC, data = sales_train)
summary(model02)
model02_test <- lm(formula = Sales.per.Visit ~ Days + CC, data = sales_test)
summary(model02_test)

cust01 <- data.frame(CC = as.factor(0), Days = 333)

predict(object = model02, newdata = cust01)

summary(model02)

X_test <- data.frame(Days = sales_test$Days, CC = sales_test$CC)
ypred <- predict(object = model02, newdata = X_test)
ytrue <- sales_test$Sales.per.Visit

#install.packages("MLmetrics")
library(MLmetrics)

MAE(y_pred = ypred, y_true = ytrue)

library(MASS)

model01_step <- stepAIC(object = model01)

summary(model01_step)



##Chapter 12
cereals <- read.csv("cereals.csv")

names(cereals)

pairs(x = cereals[, c(10, 8, 11)], pch = 16)

#install.packages("car")
library(car)

model03 <- lm(formula = Rating ~ Fiber + Potass + Sugars, data = cereals)

vif(model03)

clothes_train <- read.csv("clothing_store_PCA_training.csv")
clothes_test <- read.csv("clothing_store_PCA_test.csv")

y <- clothes_train$Sales.per.Visit
X <- clothes_train[, c(1:5)]
X_test <- clothes_test[, c(1:5)]

X_z <- as.data.frame(scale(X))
colnames(X_z) <- c("Days.since.Purchase.Z", "Purchase.Visits.Z", "Days.on.File.Z", "Days.between.Purchases.Z", "Diff.Items.Purchased.Z")

round(cor(X_z), 3)

model01 <- lm(formula = y ~ Days.since.Purchase.Z +Purchase.Visits.Z + Days.on.File.Z
              + Days.between.Purchases.Z + Diff.Items.Purchased.Z, data = X_z)

vif(model01)

#install.packages("psych")
library(psych)

pca01 <- principal(r = X_z, rotate = "varimax", nfactors = 5)

print(pca01$loadings, cutoff = 0.49)

ss.load <- c(1.566, 1.045, 1.035, 1.006, 0.348)
plot(ss.load, type = "b", main = "Plot of Eigenvalues", ylab = "Value", xlab = "Component")
abline(h = 1, lty = 2)

pca02_norot <- principal(r = X, rotate = "none", nfactors = 4)
print(pca02_norot$loadings, cutoff = 0.5)
pca02_rot <- principal(r = X, rotate = "varimax", nfactors = 4)
print(pca02_rot$loadings, cutoff = 0.5)

X_test_z <- scale(X_test)

pca02_test <- principal(r = X_test_z, rotate = "varimax", nfactors = 5)
pca02_test$loadings

pca02_test <- principal(r = X_test_z, rotate = "varimax", nfactors = 4)
print(pca02_test$loadings, cutoff = 0.5)

round(cor(pca02_rot$scores), 2)

PC1 <- pca02_rot$scores[,1]
PC2 <- pca02_rot$scores[,2]
PC3 <- pca02_rot$scores[,3]
PC4 <- pca02_rot$scores[,4]

model.pca <- lm(y ~ PC1 + PC2 + PC3 + PC4)
vif(model.pca)



##Chapter 13
sales_train <- read.csv("clothing_sales_training.csv")
sales_test <- read.csv("clothing_sales_test.csv")

logreg01 <- glm(formula = CC ~ Days + Web, data = sales_train, family = binomial)

summary(logreg01)

logreg01_test <- glm(formula = CC ~ Days + Web, data = sales_test, family = binomial)

summary(logreg01_test)

churn <- read.csv("churn.csv")

poisreg01 <- glm(formula = CustServ.Calls ~ Churn, data = churn, family = poisson)

summary(poisreg01)



##Chapter 14
churn <- read.csv("Churn_Training_File.csv")
min.churn <- subset(churn, select = c("Intl.Plan", "VMail.Plan", "CustServ.Calls", "Churn"))
min.churn$CustServ.Calls <- ordered(as.factor(min.churn$CustServ.Calls))

t1 <- table(min.churn$Intl.Plan)
t11 <- rbind(t1, round(prop.table(t1), 4 ))

colnames(t11) <- c("Intl.Plan = no ", "Intl.Plan = yes")
rownames(t11) <- c("Count", "Proportion")
t11

#install.packages("arules")
library(arules)

all.rules <- apriori(data = min.churn, parameter = list(supp = 0.01, target = "rules", conf = 0.4, minlen = 2, maxlen = 2))
inspect(head(all.rules, by = "lift", n = 10))

all.rules.ant.df <- as(as(attr(all.rules, "lhs"), "transactions"), "data.frame")
t1 <- all.rules.ant.df$items == "{Churn=True}"
t2 <- all.rules.ant.df$items == "{Churn=False}"
non.churn.ant <- abs(t1+t2-1)
good.rules <- all.rules[non.churn.ant == 1]
inspect(head(good.rules, by = "lift", n = 10))

t.csc.churn <- table(min.churn$Churn, min.churn$CustServ.Calls)
colnames(t.csc.churn) <- c("CSC = 0", "CSC = 1", "CSC = 2", "CSC = 3", "CSC = 4", "CSC = 5", "CSC = 6", "CSC = 7", "CSC = 8",
                          "CSC = 9")
rownames(t.csc.churn) <- c("Churn = False", "Churn = True")
addmargins(A = t.csc.churn, FUN = list(Total = sum), quiet = TRUE)

rules.confdiff <- apriori(data = min.churn, parameter = list(arem = "diff", aval = TRUE,
                                                             minval = 0.4, supp = 0.01, target = "rules", conf = 0.05, minlen = 2, maxlen = 2))
inspect(head(rules.confdiff, by = "lift", n = 10))

rules.confquot <- apriori(data = min.churn, parameter = list(arem = "quot", aval = TRUE, minval = 0.4, supp = 0.01, target = "rules", conf = 0.05, minlen = 2, maxlen = 2))
inspect(head(rules.confquot, by = "lift", n = 10))

rules.confquot.ant.df <- as(as(attr(rules.confquot, "lhs"), "transactions"), "data.frame")
t1 <- rules.confquot.ant.df$items == "{Churn=True}"
t2 <- rules.confquot.ant.df$items == "{Churn=False}"
non.churn.ant <- abs(t1 + t2 - 1)
good.rules.confquot <- rules.confquot[non.churn.ant == 1]
inspect(good.rules.confquot)
