﻿# 2. PythonとRの基礎
# 2･2･3 パッケージのインポート（Python）

# これはコメント！
import pandas as pd
import numpy as np
from sklearn.tree import DecisionTreeClassifier, export_graphviz

# 2･2･4 データセットの取込み（Python）
bank_train = pd.read_csv("bank_marketing_training.csv")
pd.crosstab(bank_train['previous_outcome'], bank_train['response'])
crosstab_01 = pd.crosstab(bank_train['previous_outcome'], bank_train['response'])
crosstab_01

# 2･2･6 データと変数の確認、加工（Python）
bank_train.loc[0]
bank_train.loc[[0, 2, 3]]
bank_train[0: 10]
bank_train['age']
bank_train[['age', 'job']]

# 2･2･7 グラフィックスの設定
bank_train['age'].plot(kind='hist')

# 3. データ準備
# 3･4･1 インデックスの付与（Python）
import pandas as pd
bank_train = pd.read_csv("bank_marketing_training.csv")
bank_train.shape
bank_train['index'] = pd.Series(range(0, 26874))
bank_train.head()

# 3･5･1 モデルに悪影響をもたらす変数値の変換（Python）
import numpy as np
bank_train['days_since_previous'] = bank_train['days_since_previous'].replace({999: np.NaN})
bank_train['days_since_previous'].plot(kind='hist', title='Histogram of Days Since Previous')

# 3･6･1 カテゴリ変数から数値変数の変換（Python）
bank_train['education_numeric'] = bank_train['education']
dict_edu = {"educatoin_numeric": {"illiterate": 0, "basic.4y": 4, "basic.6y": 6, "basic.9y": 9,
                                  "high.school": 12, "professional.course": 12,
                                  "university.degree": 16, "unknown": np.NaN}}
bank_train.replace(dict_edu, inplace=True)

# 3･7･1 数値変数の正規化（Python）
from scipy import stats
bank_train['age_z'] = stats.zscore(bank_train['age'])

# 3･8･1 外れ値の特定（Python）
bank_train.query('age_z > 3 | age_z < -3')
bank_train_outliers = bank_train.query('age_z > 3 | age_z < -3')
bank_train_sort = bank_train.sort_values(['age_z'], ascending=False)
bank_train_sort[['age', 'marital']].head(n=15)

# 4. 探索的データ解析
import pandas as pd
bank_train = pd.read_csv("bank_marketing_training.csv")
crosstab_01 = pd.crosstab(bank_train['previous_outcome'], bank_train['response'])
crosstab_01.plot(kind='bar', stacked=True)
crosstab_norm = crosstab_01.div(crosstab_01.sum(1), axis=0)
crosstab_norm.plot(kind='bar', stacked=True)

# 4･3･1 分割表の構築（Python）
crosstab_01 = pd.crosstab(bank_train['previous_outcome'], bank_train['previous'])
crosstab_02 = pd.crosstab(bank_train['response'], bank_train['previous_outcome'])
round(crosstab_02.div(crosstab_02.sum(0), axis=1) * 100, 1)

# 4･4･1 複合ヒストグラムの構築（Python）
import numpy as np
import matplotlib.pyplot as plt
bt_age_y = bank_train[bank_train.response == 'yes'][['age']].values
bt_age_n = bank_train[bank_train.response == 'no']['age'].values
plt.hist([bt_age_y, bt_age_n], bins=10, stacked=True)
plt.legend(['Response = Yes', 'Response = No'])
plt.title('Histogram of Age with Response Overlay')
plt.xlabel('Age')
plt.ylabel('Frequency')
plt.show()
(n, bins, patches) = plt.hist([bt_age_y, bt_age_n], bins=10, stacked=True)
n_table = np.column_stack((n[0], n[1]))
n_norm = n_table / n_table.sum(axis=1)[:, None]
ourbins = np.column_stack((bins[0:10], bins[1:11]))
p1 = plt.bar(x=ourbins[:, 0], height=n_norm[:, 0], width=ourbins[:, 1] - ourbins[:, 0])
p2 = plt.bar(x=ourbins[:, 0], height=n_norm[:, 1], width=ourbins[:, 1] - ourbins[:, 0], bottom=n_norm[:, 0])
plt.legend(['Response = Yes', 'Response = No'])
plt.title('Normalized Histogram of Age wit Response Overlay')
plt.xlabel('Age')
plt.ylabel('Proportion')
plt.show()

# 4･5･1 説明変数のビン化
bank_train['age_binned'] = pd.cut(x=bank_train['age'], bins=[0, 27, 60.01, 100], right=False)
crosstab_02 = pd.crosstab(bank_train['age_binned'], bank_train['response'])
crosstab_02.plot(kind='bar', stacked=True, title='Bar Graph of Age (binned) with Response Overlay')

# 5. モデル構築下準備
# 5･2･1 データ分割（Python）
import pandas as pd
from sklearn.model_selection import train_test_split
import random
bank = pd.read_csv("bank-additional.csv")
bank_train, bank_test = train_test_split(bank, test_size=0.25, random_state=7)
bank_train.columns
bank.shape
bank_train.shape
bank_test.shape

# 5･4･1 学習用データセットの均衡化（Python）
bank_train['response'].value_counts()
to_resample = bank_train.loc[bank_train['response'] == "yes"]
our_resample = to_resample.sample(n=841, replace=True)
bank_train_rebal = pd.concat([bank_train, our_resample])
bank_train_rebal['response'].value_counts()

# 6. 決定木
# 6･2･1 決定木（CART）モデルの構築（Python）
import pandas as pd
import numpy as np
import statsmodels.tools.tools as stattools
from sklearn.tree import DecisionTreeClassifier, export_graphviz
adult_tr = pd.read_csv("adult_ch6_training.csv")
y = adult_tr[["Income"]]
mar_np = np.array(adult_tr['Marital status'])
(mar_cat, mar_cat_dict) = stattools.categorical(mar_np, drop=True, dictnames=True)
mar_cat_pd = pd.DataFrame(mar_cat)
X = pd.concat((adult_tr[['Cap_Gains_Losses']], mar_cat_pd), axis=1)
X_names = ["Cap_Gains_Losses", "Divorced", "Married", "Never-Married", "Separated", "Widowed"]
y_names = ["<=50K", ">50K"]
cart01 = DecisionTreeClassifier(criterion="gini", max_leaf_nodes=5).fit(X, y)
export_graphviz(cart01, out_file="aaa.dot", feature_names=X_names, class_names=y_names)
predIncomeCART = cart01.predict(X)

# 6･3･1 決定木（C5.0）モデルの構築（Python)
c50_01 = DecisionTreeClassifier(criterion="entropy", max_leaf_nodes=5).fit(X, y)
export_graphviz(c50_01, out_file="c50_01.dot", feature_names=X_names, class_names=y_names)
c50_01.predict(X)

# 6･4･1 ランダムフォレストモデルの構築（Python)
from sklearn.ensemble import RandomForestClassifier
import numpy as np
rfy = np.ravel(y)
rf01 = RandomForestClassifier(n_estimators=100, criterion="gini").fit(X, rfy)
rf01.predict(X)

# 8.単純ベイズ分類器
# 8･5･1 単純ベイズ（Python）
import pandas as pd
import numpy as np
from sklearn.naive_bayes import MultinomialNB
import statsmodels.tools.tools as stattools
wine_tr = pd.read_csv("wine_flag_training.csv")
wine_test = pd.read_csv("wine_flag_test.csv")
t1 = pd.crosstab(wine_tr['Type'], wine_tr['Alcohol_flag'])
t1['Total'] = t1.sum(axis=1)
t1.loc['Total'] = t1.sum()
t1
t1_plot = pd.crosstab(wine_tr['Alcohol_flag'], wine_tr['Type'])
t1_plot.plot(kind='bar', stacked=True)
X_Alcohol_ind = np.array(wine_tr['Alcohol_flag'])
(X_Alcohol_ind, X_Alcohol_ind_dict) = stattools.categorical(X_Alcohol_ind, drop=True, dictnames=True)
X_Alcohol_ind = pd.DataFrame(X_Alcohol_ind)
X_Sugar_ind = np.array(wine_tr['Sugar_flag'])
(X_Sugar_ind, X_Sugar_ind_dict) = stattools.categorical(X_Sugar_ind, drop=True, dictnames=True)
X_Sugar_ind = pd.DataFrame(X_Sugar_ind)
X = pd.concat((X_Alcohol_ind, X_Sugar_ind), axis=1)
Y = wine_tr['Type']
nb_01 = MultinomialNB().fit(X, Y)
X_Alcohol_ind_test = np.array(wine_test['Alcohol_flag'])
(X_Alcohol_ind_test, X_Alcohol_ind_dict_test) = stattools.categorical(X_Alcohol_ind_test, drop=True, dictnames=True)
X_Alcohol_ind_test = pd.DataFrame(X_Alcohol_ind_test)
X_Sugar_ind_test = np.array(wine_test['Sugar_flag'])
(X_Sugar_ind_test, X_Sugar_ind_dict_test) = stattools.categorical(X_Sugar_ind_test, drop=True, dictnames=True)
X_Sugar_ind_test = pd.DataFrame(X_Sugar_ind_test)
X_test = pd.concat((X_Alcohol_ind_test, X_Sugar_ind_test), axis=1)
Y_predicted = nb_01.predict(X_test)
ypred = pd.crosstab(wine_test['Type'], Y_predicted, rownames=['Actual'], colnames=['Predicted'])
ypred['Total'] = ypred.sum(axis=1)
ypred.loc['Total'] = ypred.sum()
ypred

# 10.クラスタリング
# 10･5 PythonによるK-meansクラスタリングの実行方法
import pandas as pd
from scipy import stats
from sklearn.cluster import KMeans
wine_train = pd.read_csv("white_wine_training.csv")
X = wine_train[['alcohol', 'sugar']]
Xz = pd.DataFrame(stats.zscore(X), columns=['alcohol', 'sugar'])
kmeans01 = KMeans(n_clusters=2).fit(Xz)
cluster = kmeans01.labels_
Cluster1 = Xz.loc[cluster == 0]
Cluster2 = Xz.loc[cluster == 1]
Cluster1.describe()
Cluster2.describe()
wine_test = pd.read_csv("white_wine_test.csv")
X_test = wine_test[['alcohol', 'sugar']]
Xz_test = pd.DataFrame(stats.zscore(X_test), columns=['alcohol', 'sugar'])
kmeans_test = KMeans(n_clusters=2).fit(Xz_test)
cluster_test = kmeans_test.labels_ # Cluster Membership
Cluster1_test = Xz_test.loc[cluster_test == 0]
Cluster2_test = Xz_test.loc[cluster_test == 1]
Cluster1_test.describe()
Cluster2_test.describe()

# 11.回帰モデル
# 11･4 Pythonを用いた重回帰
import pandas as pd
import numpy as np
import statsmodels.api as sm
import sklearn.metrics as met
sales_train = pd.read_csv("clothing_sales_training.csv")
sales_test = pd.read_csv("clothing_sales_test.csv")
X = pd.DataFrame(sales_train[['CC', 'Days', 'Web']])
y = pd.DataFrame(sales_train[['Sales per Visit']])
X = sm.add_constant(X)
model01 = sm.OLS(y, X).fit()
X_test = pd.DataFrame(sales_test[['CC', 'Days', 'Web']])
y_test = pd.DataFrame(sales_test[['Sales per Visit']])
X_test = sm.add_constant(X_test)
model01_test = sm.OLS(y_test, X_test).fit()
model01_test.summary()
X = pd.DataFrame(sales_train[['CC', 'Days']])
X = sm.add_constant(X)
model02 = sm.OLS(y, X).fit()
model02.summary()
X_test = pd.DataFrame(sales_test[['CC', 'Days']])
X_test = sm.add_constant(X_test)
model02_test = sm.OLS(y_test, X_test).fit()
model02_test.summary()

# 11･6･1 モデル評価の実行方法（Python）
cust01 = np.column_stack((1, 0, 333))
model02.predict(cust01)
ypred = model02.predict(X_test)
ypred
np.sqrt(model02.scale)
ytrue = sales_test[['Sales per Visit']]
met.mean_absolute_error(y_true=ytrue, y_pred=ypred)

# 12. 次元削減
# 12･3･1 多重共線性の確認方法（Python）
import pandas as pd
import statsmodels.api as sm
import statsmodels.stats.outliers_influence as inf
cereals = pd.read_csv("cereals.csv")
X = pd.DataFrame(cereals[['Sugars', 'Fiber', 'Potass']])
X.columns
pd.plotting.scatter_matrix(X)
X = X.dropna()
X = sm.add_constant(X)
[inf.variance_inflation_factor(X.values, i) for i in range(X.shape[1])]

# 12･9 Pythonを用いた主成分分析の実行方法
import pandas as pd
import numpy as np
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
clothes_train = pd.read_csv("clothing_store_PCA_training.csv")
clothes_test = pd.read_csv("clothing_store_PCA_test.csv")
X = clothes_train.drop('Sales per Visit', 1)
X.corr()
pca01 = PCA(n_components=5)
principComp = pca01.fit_transform(X)
pca01.explained_variance_ratio_
np.cumsum(pca01.explained_variance_ratio_)

# 13.一般化線形モデル
# 13.4.1 ロジスティック回帰の実行方法（Python）
import pandas as pd
import numpy as np
import statsmodels.api as sm
from scipy import stats
sales_train = pd.read_csv("clothing_sales_training.csv")
sales_test = pd.read_csv("clothing_sales_test.csv")
X = pd.DataFrame(sales_train[['Days', 'Web']])
y = pd.DataFrame(sales_train[['CC']])
X = sm.add_constant(X)
logreg01 = sm.Logit(y, X).fit()
logreg01.summary2()
X_test = pd.DataFrame(sales_test[['Days', 'Web']])
y_test = pd.DataFrame(sales_test[['CC']])
X_test = sm.add_constant(X_test)
logreg01_test = sm.Logit(y_test, X_test).fit()
logreg01_test.summary2()

# 13･6･1 ポアソン回帰の実行方法（Python）
import pandas as pd
import numpy as np
import statsmodels.api as sm
import statsmodels.tools.tools as stattools
churn = pd.read_csv("churn.csv")
churn_ind = pd.get_dummies(churn['Churn'], drop_first=True)
X = pd.DataFrame(churn_ind)
X = sm.add_constant(X)
X.columns = ['const', 'Churn = True']
y = pd.DataFrame(churn[['CustServ.Calls']])
poisreg01 = sm.GLM(y, X, family=sm.families.Poisson()).fit()
poisreg01.summary()
